/*
 * JOnAS : Java(TM) OpenSource Application Server
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * Initial Developer : Guillaume Sauthier
 * --------------------------------------------------------------------------
 * $Id: XMLUtils.java,v 1.2 2004/02/16 12:16:42 sauthieg Exp $
 * --------------------------------------------------------------------------
*/

package com.bull.eclipse.jonas.utils.xml;

import java.io.IOException;
import java.io.InputStream;
import java.util.jar.JarFile;

import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.ParserConfigurationException;

import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.NodeList;
import org.w3c.dom.Text;
import org.xml.sax.SAXException;

import com.bull.eclipse.jonas.utils.xml.desc.AppClientDTDs;
import com.bull.eclipse.jonas.utils.xml.desc.AppClientSchemas;
import com.bull.eclipse.jonas.utils.xml.desc.EarDTDs;
import com.bull.eclipse.jonas.utils.xml.desc.EarSchemas;
import com.bull.eclipse.jonas.utils.xml.desc.EjbjarDTDs;
import com.bull.eclipse.jonas.utils.xml.desc.EjbjarSchemas;
import com.bull.eclipse.jonas.utils.xml.desc.HibernateDTDs;
import com.bull.eclipse.jonas.utils.xml.desc.JonasAppClientDTDs;
import com.bull.eclipse.jonas.utils.xml.desc.JonasAppClientSchemas;
import com.bull.eclipse.jonas.utils.xml.desc.JonasEjbjarDTDs;
import com.bull.eclipse.jonas.utils.xml.desc.JonasEjbjarSchemas;
import com.bull.eclipse.jonas.utils.xml.desc.JonasWebAppDTDs;
import com.bull.eclipse.jonas.utils.xml.desc.JonasWebAppSchemas;
import com.bull.eclipse.jonas.utils.xml.desc.WebAppDTDs;
import com.bull.eclipse.jonas.utils.xml.desc.WebAppSchemas;


/**
 * XML Utils Class. Holds methods for easier DOM parsing, XML modifications, ...
 *
 * @author Guillaume Sauthier
 */
public class XMLUtils {
    /** DOM factory */
    private static DocumentBuilderFactory factory = null;

    /**
     * J2EE namespace
     */
    private static final String J2EE_NS = "http://java.sun.com/xml/ns/j2ee";

    /**
     * JONAS namespace
     */
    private static final String JONAS_NS = "http://www.objectweb.org/jonas/ns";

	/** ejb-jar tag names */
	private static final String EJB_JAR = "ejb-jar";
	private static final String STANDARD_SESSION_BEAN = "session";    
	private static final String STANDARD_ENTITY_BEAN = "entity";
	private static final String STANDARD_MESSAGE_BEAN = "message-driven";
	private static final String STANDARD_EJB_NAME = "ejb-name";
	private static final String PK_TYPE = "prim-key-class";

    /** jonas-ejb-jar tag names */
    private static final String JONAS_EJB_JAR = "jonas-ejb-jar";
    private static final String SESSION_BEAN = "jonas-session";    
    private static final String ENTITY_BEAN = "jonas-entity";
    private static final String MESSAGE_BEAN = "jonas-message-driven";
    private static final String EJB_NAME = "ejb-name";
    private static final String SERVLET = "servlet";
    private static final String SERVLET_NAME = "servlet-name";
    private static final String JONAS_SERVICE_REF = "jonas-service-ref";
    private static final String SR_NAME = "service-ref-name";

    /**
     * Returns a new DocumentBuilderFactory.
     *
     * @param validating set the parser validating or not
     *
     * @return a new DocumentBuilderFactory.
     */
    private static DocumentBuilderFactory newFactory(boolean validating) {
		DocumentBuilderFactory factory = DocumentBuilderFactory.newInstance();
        factory.setNamespaceAware(true);
        factory.setValidating(validating);
        factory.setIgnoringElementContentWhitespace(true);
        //factory.setAttribute("http://apache.org/xml/features/validation/schema",
        //                     new Boolean(validating));
        return factory;
    }

    /**
     * Creates a new Document from a given InputStream.
     *
     * @param is the InputStream to be parsed
     *
     * @return the Document instance.
     *
     * @exception ParserConfigurationException
     * @exception SAXException
     * @exception IOException
     */
    public static Document newDocument(InputStream is, boolean validate)
        throws ParserConfigurationException, SAXException, IOException {
        //MANU if(factory == null) {
       	factory = newFactory(validate);
        //MANU }

        DocumentBuilder builder = factory.newDocumentBuilder();

        // add entity resolver
        if (factory.isValidating()) {
            JEntityResolver jer = new JEntityResolver();
            jer.addSchemas(new WebAppSchemas());
            jer.addSchemas(new JonasWebAppSchemas());
            jer.addSchemas(new JonasEjbjarSchemas());
			jer.addSchemas(new EjbjarSchemas());
            jer.addSchemas(new JonasAppClientSchemas());
			jer.addSchemas(new AppClientSchemas());
            jer.addSchemas(new EarSchemas());
            
            
            jer.addDtds(new WebAppDTDs());
			jer.addDtds(new JonasWebAppDTDs());
			jer.addDtds(new EjbjarDTDs());
			jer.addDtds(new JonasEjbjarDTDs());
			jer.addDtds(new AppClientDTDs());
			jer.addDtds(new JonasAppClientDTDs());
			jer.addDtds(new EarDTDs());
			jer.addDtds(new HibernateDTDs());
            
            builder.setEntityResolver(jer);
        }

        return builder.parse(is);
    }

    /**
     * Returns the
     * <code>session</code>/<code>entity</code>/<code>message-driven</code>
     * XML Element with given name.
     *
     * @param base <code>jonas-ejb-jar</code> Element.
     * @param bName the bean name to be found.
     *
     * @return the
     *         <code>session</code>/<code>entity</code>/<code>message-driven</code>
     *         XML Element.
     */
    public static Element getBeanElementInJOnASDesc(Element base, String bName) {
        Element found = null;

        // try Session Bean
        
        //MANU NodeList sessions = base.getElementsByTagNameNS(JONAS_NS, SESSION_BEAN);
		NodeList sessions = base.getElementsByTagName(SESSION_BEAN); //MANU

        for(int i = 0; (i < sessions.getLength()) && (found == null); i++) {
            Element session = (Element) sessions.item(i);
            //MANU NodeList names = session.getElementsByTagNameNS(JONAS_NS, EJB_NAME);
			NodeList names = session.getElementsByTagName(EJB_NAME); //MANU

            // ejb-name mandatory and unique
            Element name = (Element) names.item(0);

            if(name.getFirstChild()
                    .getNodeValue()
                    .equals(bName)) {
                found = session;
            }
        }
		// try Entity Bean
		//MANU NodeList entities = base.getElementsByTagNameNS(J2EE_NS, STANDARD_ENTITY_BEAN);
		NodeList entities = base.getElementsByTagName(STANDARD_ENTITY_BEAN); //MANU

		for(int i = 0; (i < entities.getLength()) && (found == null); i++) {
			Element entity = (Element) entities.item(i);
			//MANU NodeList names = entity.getElementsByTagNameNS(J2EE_NS, STANDARD_EJB_NAME);
			NodeList names = entity.getElementsByTagName(STANDARD_EJB_NAME); //MANU

			// ejb-name mandatory and unique
			Element name = (Element) names.item(0);

			if(name.getFirstChild()
					.getNodeValue()
					.equals(bName)) {
				found = entity;
			}
		}

		// try Message Driven Bean
		//MANU NodeList messages = base.getElementsByTagNameNS(J2EE_NS, STANDARD_MESSAGE_BEAN);
		NodeList messages = base.getElementsByTagName(STANDARD_MESSAGE_BEAN); //MANU

		for(int i = 0; (i < messages.getLength()) && (found == null); i++) {
			Element message = (Element) messages.item(i);
			//MANU NodeList names = message.getElementsByTagNameNS(J2EE_NS, STANDARD_EJB_NAME);
			NodeList names = message.getElementsByTagName(STANDARD_EJB_NAME); //MANU

			// ejb-name mandatory and unique
			Element name = (Element) names.item(0);

			if(name.getFirstChild()
					.getNodeValue()
					.equals(bName)) {
				found = message;
			}
		}

		return found;
	}

		/**
		 * Returns the
		 * <code>session</code>/<code>entity</code>/<code>message-driven</code>
		 * XML Element with given name.
		 *
		 * @param base <code>ejb-jar</code> Element.
		 * @param bName the bean name to be found.
		 *
		 * @return the
		 *         <code>session</code>/<code>entity</code>/<code>message-driven</code>
		 *         XML Element.
		 */
		public static Element getBeanElementInStandardDesc(Element base, String bName) {
			Element found = null;

			// try Session Bean
        
			//MANU NodeList sessions = base.getElementsByTagNameNS(J2EE_NS, STANDARD_SESSION_BEAN);
			NodeList sessions = base.getElementsByTagName(STANDARD_SESSION_BEAN); //MANU
			
			for(int i = 0; (i < sessions.getLength()) && (found == null); i++) {
				Element session = (Element) sessions.item(i);
				//MANU NodeList names = session.getElementsByTagNameNS(J2EE_NS, STANDARD_EJB_NAME);
				NodeList names = session.getElementsByTagName(STANDARD_EJB_NAME); //MANU

				// ejb-name mandatory and unique
				Element name = (Element) names.item(0);
				
				if(name.getFirstChild()
						.getNodeValue()
						.equals(bName)) {
					found = session;
				}
			}

        // try Entity Bean
        //MANU NodeList entities = base.getElementsByTagNameNS(J2EE_NS, STANDARD_ENTITY_BEAN);
		NodeList entities = base.getElementsByTagName(STANDARD_ENTITY_BEAN); //MANU
		
        for(int i = 0; (i < entities.getLength()) && (found == null); i++) {
            Element entity = (Element) entities.item(i);
            //MANU NodeList names = entity.getElementsByTagNameNS(J2EE_NS, STANDARD_EJB_NAME);
			NodeList names = entity.getElementsByTagName(STANDARD_EJB_NAME); //MANU
			
            // ejb-name mandatory and unique
            Element name = (Element) names.item(0);

            if(name.getFirstChild()
                    .getNodeValue()
                    .equals(bName)) {
                found = entity;
            }
        }

        // try Message Driven Bean
        NodeList messages = base.getElementsByTagNameNS(J2EE_NS, STANDARD_MESSAGE_BEAN);

        for(int i = 0; (i < messages.getLength()) && (found == null); i++) {
            Element message = (Element) messages.item(i);
            NodeList names = message.getElementsByTagNameNS(J2EE_NS, STANDARD_EJB_NAME);

            // ejb-name mandatory and unique
            Element name = (Element) names.item(0);

            if(name.getFirstChild()
                    .getNodeValue()
                    .equals(bName)) {
                found = message;
            }
        }

        return found;
    }

    /**
     * Returns the matching <code>servlet</code> XML Element with given name.
     *
     * @param base <code>web-app</code> Element.
     * @param sName the servlet name to be found.
     *
     * @return the matching <code>servlet</code> XML Element.
     */
    public static Element getServletElement(Element base, String sName) {
        Element found = null;

        // Search servlet List
        //MANU NodeList servlets = base.getElementsByTagNameNS(J2EE_NS, SERVLET);
	
		NodeList servlets = base.getElementsByTagName(SERVLET); //MANU
		
        for(int i = 0; (i < servlets.getLength()) && (found == null); i++) {
            Element servlet = (Element) servlets.item(i);
            NodeList names =
                servlet.getElementsByTagName(SERVLET_NAME); //MANU

            // servlet-name mandatory and unique
            Element name = (Element) names.item(0);

            if(name.getFirstChild()
                    .getNodeValue()
                    .equals(sName)) {
                found = servlet;
            }
        }

        return found;
    }

    /**
     * Returns the <code>jonas-service-ref</code> XML Element with given name.
     *
     * @param base <code>web-app</code> / bean Element containing
     *        <code>jonas-service-ref</code> Element(s).
     * @param srName the service-ref name to be found.
     *
     * @return the <code>jonas-service-ref</code> XML Element.
     */
    public static Element getJonasServiceRef(Element base, String srName) {
        Element found = null;

        // Search jonas-service-ref list
        NodeList jsrs =
            base.getElementsByTagNameNS(JONAS_NS, JONAS_SERVICE_REF);

        for(int i = 0; (i < jsrs.getLength()) && (found == null); i++) {
            Element jsr = (Element) jsrs.item(i);
            NodeList names = jsr.getElementsByTagNameNS(JONAS_NS, SR_NAME);

            // service-ref-name mandatory and unique
            Element name = (Element) names.item(0);

            if(name.getFirstChild()
                    .getNodeValue()
                    .equals(srName)) {
                found = jsr;
            }
        }

        return found;
    }

    /**
     * default application.xml contains a fake ejb module needed to be parsed
     * without error but not needed for a normal application. Than we remove it.
     *
     * @param doc application.xml document
     */
    public static void cleanDummyApplication(Document doc) {
        Element root = doc.getDocumentElement();
        NodeList nl = root.getElementsByTagNameNS(J2EE_NS, "module");
        root.removeChild(nl.item(0));
    }

    /**
     * Add an ejb module in an application Document
     *
     * @param app application.xml Document
     * @param ejbjar EJBJar archive
     */
    public static void addEjb(Document app, JarFile ejbjar) {
        Element module = app.createElementNS(J2EE_NS, "module");
        Element ejb = app.createElementNS(J2EE_NS, "ejb");
        Text ejbText = app.createTextNode(ejbjar.getName());
        ejb.appendChild(ejbText);
        module.appendChild(ejb);

        insertModule(app, module);
    }

    /**
     * Add a client module in an application Document
     *
     * @param app application.xml Document
     * @param client Client archive
     */
    public static void addClient(Document app, JarFile client) {
        Element module = app.createElementNS(J2EE_NS, "module");
        Element clt = app.createElementNS(J2EE_NS, "java");
        Text cltText = app.createTextNode(client.getName());
        clt.appendChild(cltText);
        module.appendChild(clt);

        insertModule(app, module);
    }

    /**
     * Add an web module in an application Document
     *
     * @param app application.xml Document
     * @param webapp WebApp archive
     * @param ctx context-root
     */
    public static void addWebApp(Document app, JarFile webapp, String ctx) {
        Element module = app.createElementNS(J2EE_NS, "module");
        Element web = app.createElementNS(J2EE_NS, "web");
        Element webUri = app.createElementNS(J2EE_NS, "web-uri");
        Element context = app.createElementNS(J2EE_NS, "context-root");

        Text webText = app.createTextNode(webapp.getName());
        Text ctxText = app.createTextNode(ctx);
        webUri.appendChild(webText);
        context.appendChild(ctxText);

        web.appendChild(webUri);
        web.appendChild(context);

        module.appendChild(web);

        insertModule(app, module);
    }

    /**
     * Insert a module in application Document
     *
     * @param app application.xml Document
     * @param module module Element
     */
    private static void insertModule(Document app, Element module) {
        Element application = app.getDocumentElement();
        Element first = findFirstSecurityRole(application);
        application.insertBefore(module, first);
    }

    /**
     * return the first found security-role Element (or null if not found)
     *
     * @param app application.xml Document
     *
     * @return the first found security-role Element (or null if not found)
     */
    private static Element findFirstSecurityRole(Element app) {
        NodeList nl = app.getElementsByTagNameNS(J2EE_NS, "security-role");

        if(nl.getLength() == 0) {
            return null;
        } else {
            return (Element) nl.item(0);
        }
    }
    
}
